#### File: network.R
#### Author: Richard Longland (2007)
#### Description: Plot the output of a network calculation
#### Usage: See README file.

# Libraries
#library(sna)	# for plotting arrows

# A few preferences
Title = "xxx"
TitleScale = 1.4

TitlePos = 0.9 # In y-direction from bottom (max=1)

LW = 0.7 	# Line width in fraction of default

ArrScale = 0.5	# Scale for arrow thickness
headscale = 0.4   # Arrow head size scale
powScale = 1.4	# Arrow scaling parameter (width=score^powScale)

textScale = 3.0

keyOffset = c(5,4)  # key offset (x,y) from the lower right
keyScale = 1.0
keytextScale = 0.4

minZ <- 0       # Minimum Z for plotting (default=0)
maxZ <- Inf     # Max Z for plotting (set 'Inf' for auto)
minN <- 0       # Same for N
maxN <- Inf

##minZ <- 15       # Minimum Z for plotting (default=0)
##maxZ <- 22     # Max Z for plotting (set 'Inf' for auto)
##minN <- 11       # Same for N
##maxN <- 25


FluxBreaks <- c(1e1,1e4,1e7,1e10,1e12) # defines arrow thickness breaks
## FluxBreaks <- c(1e1,1e3,1e6,1e9,1e12) # defines arrow thickness breaks

## Extra nuclei boxes to draw that aren't included in the network
## Comment out with '#' to ignore
## input c(Z,N)
ExtraNuc <- rbind(c(5,4),   # 9B
                  c(4,4))   # 8Be

margins <- c(0,0,0,0)     # bottom, left, top, right
#######################################################
## new default plotting parameters
mypdf <- function(file="output.pdf",...)
  {
    grDevices::cairo_pdf(file=file,...)
    par(cex.axis=1.3, cex.lab=1.5,   # Font sizes
        las=1,                       # Always horisontal text
        lwd=2,                       # Line width
        mar=c(5,5,3,2)+0.1,          # Margins
        pch=19,                      # Point type (solid circles)
        tcl=0.5,
        mgp=c(3,0.5,0))     
  }

######################################################################
## Function to draw the key
draw.key<-function(){
	# Position in lower right of picture
	xpos <- plotMax - keyOffset[1]
	ypos <- keyOffset[2]

	rect(xpos-0.5,ypos-0.5,xpos+0.5,ypos+0.5,lwd=LW*par("lwd"))
	# Draw the arrows
	dy<-keyScale*c(-2,-1,1,1.4,2,2,1,0,-1,-1.4,0,-2)
	dx<-keyScale*c(-1,-2,-1,0,1,2,2,1.4,1,0,-1.4,-2)
	for(i in 1:length(dx))plot.arrow(xpos,ypos,xpos+dx[i],ypos+dy[i],
                                         width=ArrScale,hscale=headscale)
	
	# Draw the key
	dy <- 1.2*keyScale^0.8*c(-2.3,-1.05,1.3,1.8,2.3,2.3,1.1,0,-1.3,-1.8,0,-2.3)
	dx <- 1.2*keyScale^0.8*c(-1,-2.6,-1.5,0,1,2.2,2.5,2.0,1.5,0,-2.0,-2.2)
	textposx<-xpos+dx
	textposy<-ypos+dy
#	keytext=c("(n,\\*a)","(p,\\*a)","(\\*b\\a-\\*n),(p,n)",
#          "(p,\\*g)","(\\*a,n)","(\\*a,\\*g)","(\\*a,p)","(n,\\*g)",
#          "(\\*b\\sp+\\ep\\*n),(n,p)","(\\*g,p)","(\\*g,n)")

        keytext=c("(n,alpha)",#paste("(n*","','","*alpha)"),
          "(p,alpha)",
          "(beta^{-phantom()}*nu),(p,n)",
          "(p,gamma)","(alpha,n)","(alpha,gamma)","(alpha,p)","(n,gamma)",
          "(beta^{phantom()+phantom()}*nu),(n,p)",
          "(gamma~phantom(),p)","(gamma~phantom(),n)",
          "(gamma~phantom(),alpha)")
        keytext <- gsub(",", "*symbol(\"\\\\54\")*",keytext)
        keytext=paste("phantom()",keytext,sep="")
        keytext[3] <- "atop((beta^{-phantom()}*nu),(p*symbol(\"\\54\")*n))"
        keytext[9] <- "atop((beta^{phantom()+phantom()}*nu),(n*symbol(\"\\54\")*p))"


	text(textposx,textposy,labels=parse(text=keytext),
             cex=0.4*textScale*keytextScale)
}
#######################################################
# Draw arrows
plot.arrow <- function(x1,y1,x2,y2,width,hscale=headscale){
  ## first draw the line
  ##  cat(x1,y1,x2,y2,"\n")

  ## --- The head, point at (0,0) first ---
  a <- 0.5*hscale*powScale    # length
  b <- 0.5*hscale*powScale    # inner length
  c <- 0.2*hscale*powScale    # width
  ## The arrow head for 0 degrees
  arr.coords <- cbind(c(0,-a,-b,-a,0)+0.25*a,c(0,c,0,-c,0))
#  arr.coords <- cbind(c(0,-a,-b,-a,0),c(0,c,0,-c,0))
  arr.head <- function(x,y,ang,scale){  # function to make individual head
    ## Now rotate
    arr.coords <- (0.5*scale)^0.5*cbind(cos(ang)*arr.coords[,1]-sin(ang)*arr.coords[,2],
                        sin(ang)*arr.coords[,1]+cos(ang)*arr.coords[,2])
    ## and translate to the end of the line
    arr.coords <- cbind(arr.coords[,1]+x,arr.coords[,2]+y)
    arr.coords
  }

  ## The angle for each flow line
  angle <- atan((y2-y1)/(x2-x1))
  angle[x2<x1] <- pi+angle[x2<x1]
  ## Now do the drawing part (every four)
  for(i in 1:length(x1)){
    #cat(x1[i],y1[i],x2[i],y2[i],"\n")
    if(width[i]>0){
      lines(c(x1[i],x2[i]),c(y1[i],y2[i]),lwd=width[i])
      polygon(arr.head(x2[i],y2[i],angle[i],scale=width[i]),col="black",border=F)
    }
  }
}

#######################################################
#######################################################

# open the postscript file
mypdf(file="flux.pdf",width=10,height=10,onefile=F)
oldpar <- par(mar=c(0,0,0,0),omi=margins)

Names<-c("n","H","He","Li","Be","B","C","N","O","F","Ne","Na","Mg","Al","Si",
	"P","S","Cl","Ar","K","Ca","Sc","Ti","V","Cr","Mn","Fe","Co","Ni",
         "Cu","Zn","Ga","Ge","As","Se","Br","Kr","Rb","Sr","Y","Zr","Nb","Mo",
         "Tc","Ru","Rh","Pd","Ag","Cd","In","Sn","Sb","Te","I",
         "Xe","Cs","Ba","La","Ce","Pr","Nd","Pm","Sm","Eu","Gd",
         "Tb","Dy","Ho","Er","Tm","Yb","Lu","Hf","Ta","W","Re",
         "Os","Ir","Pt","Au","Hg","Tl","Pb","Bi","Po","At","Rn",
         "Fr","Ra","Ac","Th","Pa","U","Np","Pu","Am","Cm","Bk",
         "Cf","Es","Fm")

# Load the fortran code needed to analyse the flux data
if(!is.loaded("fluxSub"))
  dyn.load("rflux.so") #,Local=FALSE)
.Fortran("fluxSub")
dyn.unload("rflux.so")

# Read in Flux data from file
# Read the number of Nuclei
nNuc <- scan("rflux.out",n=1)

# Read the nuclei as (index, name, A, Z, N)
# note: try read.fortran? or read.fwf
Nuclei <- read.table("rflux.out",nrows=nNuc,skip=1,sep=",")

# Read in the reactions
React <- read.table("rflux.out",skip=2+nNuc,sep=",")


Nallow <- Nuclei[,4]<=maxZ & Nuclei[,4]>=minZ
Rallow <- React[,3]<=maxZ & React[,3]>=minZ
Nallow <- Nallow & Nuclei[,5]<=maxN & Nuclei[,5]>=minN
Rallow <- Rallow & React[,2]<=maxN & React[,2]>=minN

Nuclei <- Nuclei[Nallow,]
React <- React[Rallow,]

## React[,6] is the line width. Replace this in here using React[,7],
## the actual net flux
React[,7] <- as.double(gsub("D","E",React[,7]))
MaxFlux <- max(React[,7])

React[,6] <- length(FluxBreaks)
for(i in 1:length(FluxBreaks)){
  chosen <- React[,7] <= MaxFlux/FluxBreaks[i]
  React[chosen,6] <- length(FluxBreaks)-i
}


# Now make a plot
plotMax<-max(Nuclei[,4],Nuclei[,5])
plotMin <- min(Nuclei[,4],Nuclei[,5])
##plotMax <- min(plotMax,max(maxZ,Nuclei[Nuclei[,4]<maxZ,5])+1)
plot(c(plotMin-1,plotMax+1),c(plotMin-1,plotMax+1),type="n", xlab="", ylab="",
     axes=FALSE,asp=1)
## Make the title
text(x=grconvertX(0.5,from="ndc",to="user"),
     y=grconvertY(TitlePos,from="ndc",to="user"),
     labels=Title,cex=2.0*TitleScale,font=2)

rect(Nuclei[,5]-0.5,Nuclei[,4]-0.5,Nuclei[,5]+0.5,Nuclei[,4]+0.5,
     lwd=LW*par("lwd"))
## Add extra squared if required
if(exists("ExtraNuc"))
  rect(ExtraNuc[,2]-0.5,ExtraNuc[,1]-0.5,ExtraNuc[,2]+0.5,ExtraNuc[,1]+0.5,
       lwd=LW*par("lwd"))

# Mark off, the stable nuclei
stabNuc<-read.table("stableslist.dat",header=FALSE)
# Only go as far as the maximum in the reaction file
for(i in 1:length(stabNuc[,2])){
  if(stabNuc[i,2] <= max(Nuclei[,4])
     & stabNuc[i,2] >= min(Nuclei[,4])
     & stabNuc[i,3] <= max(Nuclei[,5])
     & stabNuc[i,3] >= min(Nuclei[,5])){
    ## density=NA makes filled rectangles, numeric values give hatching
    {rect(stabNuc[i,3]-0.5,stabNuc[i,2]-0.5,stabNuc[i,3]+0.5,stabNuc[i,2]+0.5,
          density=NA,lwd=LW*par("lwd"),col="gray70",border="black")}
  }
}
# Make the labels
# Find every Z used
laby<-as.double(levels(factor(Nuclei$V4)))
# Find minimum N for each one, and subtract 1
labx<-matrix(tapply(Nuclei$V5,factor(Nuclei$V4),min)-1.1)[,1]
# Print the labels
#text(x=labx,y=laby,labels=Names[(1+minZ):(length(laby)+minZ)],
#     cex=0.4*textScale)
text(x=labx,y=laby,labels=Names[laby+1],
     cex=0.4*textScale)
# Now label the n's (only want odd ones)
labx<-as.double(levels(factor(Nuclei$V5)))[2*1:(nlevels(factor(Nuclei$V5))/2)]
laby<-matrix(tapply(Nuclei$V4,factor(Nuclei$V5),min)-1
             )[2*1:(nlevels(factor(Nuclei$V5))/2),1]
text(x=labx,y=laby,labels=labx, #vfont=c("sans serif","bold"),
     cex=0.4*textScale)


# Draw arrows for reactions
plot.arrow(React[,2],React[,3],React[,4],React[,5],
           width=ArrScale*React[,6]^powScale)

#Draw the key
#draw.key()

# output the postscript file
dev.off()



